function [fout, isterminal, direction] = event(t,v,n_diff,parameters)
%fout = event(t,v,n,parameters)
%   Event function that defines either calving front (if terminus is grounded) or
%   grounding line (if terminus is floating)
%   Input arguments
%       t:  time (redundant, used for code compatibility with
%           shoot_parameters only)
%       v:  (xi,Psi,Q) state variable
%       n_diff: Order of differentiation (compatible with shoot_parameters)
%               n_diff = 0 causes 'event' to output the event function
%               itself
%               n_diff = 1 causes 'event' to output the gradient of the
%               event function with respect to v
%               n_diff = 10 causes 'event' to output the gradient of the
%               event function with respect to the parameters specified in
%               parameters.type. parameters.type is a logical indexing
%               vector from [C,M,Lambda,H_g] into a parameter vector v_par.
%               For instance, if parameters.type = [3; 2], the v_par =
%               [Lambda; M], and n_diff = 10 causes 'event' to output the
%               gradient of the event function with respect to [Lambda; M].
%       parameters: Parameter structure containing the physical parameters
%           m,n,r,C,M,Lambda,H_g as fields, plus additional fields
%               bc_type: String, with possible options being
%                   'grounded calving', 'flotating_calving', 'grounding'
%                   These specify that 'event' should represent the
%                   following boundary conditions:
%                   'grounded calving': This is the grounded calving face
%                   stress condition
%                       Psi*xi-1/8-r*H_g^2*xi^(-2*n^2/(n+1)^2) = 0;
%                   'grounding': This is the grounding thickness condition
%                       xi - H_g^(-(n+1)^2/n^2)
%                   'flotation': This is the stress condition at flotation
%                      Psi*xi - (1-r)/8;
%               type:   Specifies which parameters are being varied (see n_diff
%                   above)
%           Note that 'parameters' here is 'parameters.model' in
%           boundary_layer_shoot

xi = v(1);
Psi = v(2);
Q = v(3);

n = parameters.n;
r = parameters.r;
H_g = parameters.H_g;

if ~isfield(parameters,'bc_type')
    bc_type = 'grounded calving';
else
    bc_type = parameters.bc_type;
end
switch bc_type
    case 'grounded calving'
        switch n_diff
            case 0
                fout = Psi*xi-1/8+r*H_g^2*xi^(2*n^2/(n+1)^2)/8;
                if nargout > 1
                    isterminal = true;
                    direction = 0;
                end
            case 1
                fout = zeros(3,1);
                fout(1) = Psi+(2*n^2/(n+1)^2)*r*H_g^2*xi^(2*n^2/(n+1)^2-1)/8;
                fout(2) = xi;
            case 10
                type = parameters.type;
                fout = zeros(length(type),1);
                for ii = 1:length(type)
                    switch type(ii)
                        case 4
                            fout(ii) = r*H_g*xi^(2*n^2/(n+1)^2)/4;
                    end
                end
        end
    case 'flotation'
        switch n_diff
            case 0
                fout = Psi*xi-(1-r)/8;
                if nargout > 1
                    isterminal = true;
                    direction = 0;
                end
            case 1
                fout = zeros(3,1);
                fout(1) = Psi;
                fout(2) = xi;
            case 10
                type = parameters.type;
                fout = zeros(length(type),1);
        end        
    case 'grounding'
        switch n_diff
            case 0
                fout = Q*xi^(-n^2/(n+1)^2)-H_g;
                if nargout > 1
                    isterminal = true;
                    direction = 0;
                end
            case 1
                fout = zeros(3,1);
                fout(1) = (-n^2/(n+1)^2)*Q*xi^(-n^2/(n+1)^2-1);
                fout(3) = xi^(-n^2/(n+1)^2);
            case 10
                type = parameters.type;
                fout = zeros(length(type),1);
                for ii = 1:length(type)
                    switch type(ii)
                        case 4
                            fout(ii) = -1;
                    end
                end
        end
    otherwise
        error('option not implemented')
end

end

