function fout = outlet_evolve(x,v,n_diff,parameters)
%%fout = evolve(x,v,n,parameters)
%   Right-hand side of the dynamical system representing the steady outlet
%   glacier problem
%   Input arguments
%       x:  position x
%       v:  (h,sigma,q,x_prime) state variable
%       n_diff: Order of differentiation (compatible with shoot_parameters)
%               n_diff = 0 causes 'evolution' to output the evolution function
%               itself
%               n_diff = 1 causes 'evolve' to output the Jacobian of the
%               evolution function with respect to v
%               n_diff = 2 causes 'evolve' to output the second derivative
%               array of the evolution function with respect to v
%               n_diff = 10 causes 'evolve' to output the gradient of the
%               evolution function with respect to the parameters specified in
%               parameters.type. parameters.type is a logical indexing
%               vector from [a_0;C_0;M_0;Lambda_0;x_f]
%               into a parameter vector v_par.
%               For instance, if parameters.type = [3; 2], the v_par =
%               [M_0;C_0], and n_diff = 10 causes 'event' to output the
%               gradient of the event function with respect to [Lambda; M].
%               n_diff = 11 causes 'evolve' to output the mixed second
%               derivative array of the evolution function with respect to
%               v and the parameters specified in parameters.type.
%       parameters: Parameter structure containing the fields
%           m,n,r,epsilon,x_f,C_0,M_0,a_0,theta,latdrag:    parameter values in outlet
%                               glacier model, see outlet_shoot for details
%           bfun, wfun, accfun, meltfun:    
%                               function handles for B, w, a and M and
%                               their derivatives
%               type:   Specifies which parameters are being varied (see n_diff
%                       above)
%           Note that 'parameters' here is 'parameters.model' in
%           outlet_shoot
%   output arument:


h = v(1);
sigma = v(2);
q = v(3);
x_prime = v(4);

n = parameters.n;
m = parameters.m;
r = parameters.r;
epsilon = parameters.epsilon;
C_0 = parameters.C_0;
M_0 = parameters.M_0;
a_0 = parameters.a_0;
theta = parameters.theta;

x_f = parameters.x_f;
w = parameters.Wfun(x_prime,0,parameters);
w_x = parameters.Wfun(x_prime,1,parameters);
w_xx = parameters.Wfun(x_prime,2,parameters);
w_xxx = parameters.Wfun(x_prime,3,parameters);

B_x = parameters.Bfun(x_prime,1,parameters);
B_xx = parameters.Bfun(x_prime,2,parameters);
B_xxx = parameters.Bfun(x_prime,3,parameters);

m_0 = M_0*parameters.meltfun(x-x_f,0,parameters);
m_0_x = M_0*parameters.meltfun(x-x_f,1,parameters);
m_0_xx = M_0*parameters.meltfun(x-x_f,2,parameters);

mu = epsilon^(-n/(n+1))*m_0;
mu_x = epsilon^(-n/(n+1))*m_0_x;
mu_xx = epsilon^(-n/(n+1))*m_0_xx;

gamma = epsilon^((n^2*(1+m)-n)/(n+1)^2)*C_0;
b_x = epsilon^(n^2/(n+1)^2)*B_x;
b_xx = epsilon^(n^2/(n+1)^2)*B_xx;
b_xxx = epsilon^(n^2/(n+1)^2)*B_xxx;

a = a_0*parameters.accfun(x,0,parameters) - (1-theta)*mu;
a_x = a_0*parameters.accfun(x,1,parameters) - (1-theta)*mu_x;
a_xx = a_0*parameters.accfun(x,2,parameters) - (1-theta)*mu_xx;

%parameter that permits lateral drag to be switched off, for comparison
%with earlier MISMIP-type results
if ~isfield(parameters,'latdrag')
    latdrag = 1;
else
    latdrag = parameters.latdrag;
end


switch n_diff
    case 0
        fout = zeros(4,1);
        fout(1) = -h^2*w*abs(sigma)^(n-1)*sigma/q + h*w*a/q - h*w_x/w;
        fout(2) = latdrag*abs(q)^(1/n-1)*q/(4*epsilon*h^(1/n)*w^(2/n+1)) ...
            + theta*gamma*abs(q)^(m-1)*q/(4*epsilon*h^(m+1)*w^m) ...
            + theta*(fout(1)+b_x)/(4*epsilon) ...
            +(1-theta)*(1-r)*fout(1)/(4*epsilon) ...
            + h*w*abs(sigma)^(n+1)/q - a*w*sigma/q + sigma*w_x/w;
        fout(3) = a*w;
        fout(4) = 1;
        
    case 1
        fout = zeros(4,4);
        
        fout(1,1) = -2*h*w*abs(sigma)^(n-1)*sigma/q + w*a/q - w_x/w;
        fout(1,2) = -n*h^2*w*abs(sigma)^(n-1)/q;
        fout(1,3) = h^2*w*abs(sigma)^(n-1)*sigma/q^2 - h*w*a/q^2;
        fout(1,4) = -h^2*w_x*abs(sigma)^(n-1)*sigma/q + h*w_x*a/q + h*w*a_x/q - h*w_xx/w + h*w_x^2/w^2;
        
        fout(2,1) =  -latdrag*1/n*abs(q)^(1/n-1)*q/(4*epsilon*h^(1/n+1)*w^(2/n+1)) ...
            - (m+1)*theta*gamma*abs(q)^(m-1)*q/(4*epsilon*h^(m+2)*w^m) ...
            + theta*fout(1,1)/(4*epsilon) ...
            +(1-theta)*(1-r)*fout(1,1)/(4*epsilon) ...
            + w*abs(sigma)^(n+1)/q;
        fout(2,2) = theta*fout(1,2)/(4*epsilon) ...
            +(1-theta)*(1-r)*fout(1,2)/(4*epsilon) ...
            + (n+1)*h*w*abs(sigma)^n/q - a*w/q + w_x/w;
        fout(2,3) = latdrag*1/n*abs(q)^(1/n-1)/(4*epsilon*h^(1/n)*w^(2/n+1)) ...
            + m*theta*gamma*abs(q)^(m-1)/(4*epsilon*h^(m+1)*w^m) ...
            + theta*fout(1,3)/(4*epsilon) ...
            +(1-theta)*(1-r)*fout(1,3)/(4*epsilon) ...
            - h*w*abs(sigma)^(n+1)/q^2 + a*w*sigma/q^2;
        fout(2,4) = -latdrag*(2/n+1)*abs(q)^(1/n-1)*q/(4*epsilon*h^(1/n)*w^(2/n+2))*w_x ...
            - m*theta*gamma*abs(q)^(m-1)*q/(4*epsilon*h^(m+1)*w^(m+1))*w_x ...
            + theta*(fout(1,4)+b_xx)/(4*epsilon) ...
            +(1-theta)*(1-r)*fout(1,4)/(4*epsilon) ...
            + h*w_x*abs(sigma)^(n+1)/q - a_x*w*sigma/q - a*w_x*sigma/q + sigma*w_xx/w - sigma*w_x^2/w^2;
        
        fout(3,4) = a_x*w + a*w_x;
               
    case 2
        
        fout = zeros(4,4,4);
        
        fout(1,1,1) = -2*w*abs(sigma)^(n-1)*sigma/q;
        fout(1,1,2) =  -n*2*h*w*abs(sigma)^(n-1)/q;
        fout(1,2,1) = fout(1,1,2);
        fout(1,1,3) = 2*h*w*abs(sigma)^(n-1)*sigma/q^2 - w*a/q^2;
        fout(1,3,1) = fout(1,1,3);
        fout(1,1,4) = -2*h*w_x*abs(sigma)^(n-1)*sigma/q + w_x*a/q + w*a_x/q - w_xx/w + w_x^2/w^2;
        fout(1,4,1) = fout(1,1,4);
        fout(1,2,2) = -n*(n-1)*h^2*w*abs(sigma)^(n-3)*sigma/q;
        fout(1,2,3) = n*h^2*w*abs(sigma)^(n-1)/q^2;
        fout(1,3,2) = fout(1,2,3);
        fout(1,2,4) = -n*h^2*w_x*abs(sigma)^(n-1)/q;
        fout(1,4,2) = fout(1,2,4);
        fout(1,3,3) = -2*h^2*w*abs(sigma)^(n-1)*sigma/q^3 + 2*h*w*a/q^3;
        fout(1,3,4) =  h^2*w_x*abs(sigma)^(n-1)*sigma/q^2 - h*w_x*a/q^2 - h*w*a_x/q^2;
        fout(1,4,3) = fout(1,3,4);
        fout(1,4,4) =  -h^2*w_xx*abs(sigma)^(n-1)*sigma/q + h*w_xx*a/q + 2*h*w_x*a_x/q + h*w*a_xx/q - h*w_xxx/w + h*w_xx*w_x/w^2 - 2*h*w_x^3/w^3 + 2*h*w_x*w_xx/w^2;
         
        fout(2,1,1) =  latdrag*1/n*(1/n+1)*abs(q)^(1/n-1)*q/(4*epsilon*h^(1/n+2)*w^(2/n+1)) ...
            + (m+1)*(m+2)*theta*gamma*abs(q)^(m-1)*q/(4*epsilon*h^(m+3)*w^m) ...
            + theta*fout(1,1,1)/(4*epsilon) ...
            +(1-theta)*(1-r)*fout(1,1,1)/(4*epsilon);
        fout(2,1,2) = theta*fout(1,1,2)/(4*epsilon) ...
            +(1-theta)*(1-r)*fout(1,1,2)/(4*epsilon) ...
            + (n+1)*w*abs(sigma)^n/q;
        fout(2,2,1) = fout(2,1,2);
        fout(2,1,3) = -latdrag*1/n^2*abs(q)^(1/n-1)/(4*epsilon*h^(1/n+1)*w^(2/n+1)) ...
            - (m+1)*m*theta*gamma*abs(q)^(m-1)/(4*epsilon*h^(m+2)*w^m) ...
            + theta*fout(1,1,3)/(4*epsilon) ...
            +(1-theta)*(1-r)*fout(1,1,3)/(4*epsilon) ...
            - w*abs(sigma)^(n+1)/q^2;       
        fout(2,3,1) = fout(2,1,3);
        fout(2,1,4) =  latdrag*(2/n+1)*1/n*abs(q)^(1/n-1)*q/(4*epsilon*h^(1/n+1)*w^(2/n+2))*w_x ...
            + (m+1)*m*theta*gamma*abs(q)^(m-1)*q/(4*epsilon*h^(m+2)*w^(m+1))*w_x ...
            + theta*fout(1,1,4)/(4*epsilon) ...
            +(1-theta)*(1-r)*fout(1,1,4)/(4*epsilon) ...
            + w_x*abs(sigma)^(n+1)/q;
        fout(2,4,1) = fout(2,1,4);
        
        fout(2,2,2) = theta*fout(1,2,2)/(4*epsilon) ...
            +(1-theta)*(1-r)*fout(1,2,2)/(4*epsilon) ...
            + (n+1)*n*h*w*abs(sigma)^(n-2)*sigma/q;
        fout(2,2,3) =  theta*fout(1,2,3)/(4*epsilon) ...
            +(1-theta)*(1-r)*fout(1,2,3)/(4*epsilon) ...
            - (n+1)*h*w*abs(sigma)^n/q^2 + a*w/q^2;
        fout(2,3,2) = fout(2,2,3);
        fout(2,2,4) = theta*fout(1,2,4)/(4*epsilon) ...
            +(1-theta)*(1-r)*fout(1,2,4)/(4*epsilon) ...
            + (n+1)*h*w_x*abs(sigma)^n/q - a_x*w/q - a*w_x/q + w_xx/w - w_x^2/w^2;
        fout(2,4,2) = fout(2,2,4);

       fout(2,3,3) = latdrag*1/n*(1/n-1)*abs(q)^(1/n-3)*q/(4*epsilon*h^(1/n)*w^(2/n+1)) ...
            + m*(m-1)*theta*gamma*abs(q)^(m-3)*q/(4*epsilon*h^(m+1)*w^m) ...
            + theta*fout(1,3,3)/(4*epsilon) ...
            +(1-theta)*(1-r)*fout(1,3,3)/(4*epsilon) ...
            + 2*h*w*abs(sigma)^(n+1)/q^3 - 2*a*w*sigma/q^3;
        fout(2,3,4) = -latdrag*(2/n+1)*1/n*abs(q)^(1/n-1)/(4*epsilon*h^(1/n)*w^(2/n+2))*w_x ...
            - m^2*theta*gamma*abs(q)^(m-1)/(4*epsilon*h^(m+1)*w^(m+1))*w_x ...
            + theta*fout(1,3,4)/(4*epsilon) ...
            +(1-theta)*(1-r)*fout(1,3,4)/(4*epsilon) ...
            - h*w_x*abs(sigma)^(n+1)/q^2 + a_x*w*sigma/q^2+ a*w_x*sigma/q^2 ;
        fout(2,4,3) = fout(2,3,4);
        
        fout(2,4,4) = latdrag*(2/n+1)*(2/n+2)*abs(q)^(1/n-1)*q/(4*epsilon*h^(1/n)*w^(2/n+3))*w_x^2 ...
            -latdrag*(2/n+1)*abs(q)^(1/n-1)*q/(4*epsilon*h^(1/n)*w^(2/n+2))*w_xx ...
            + m*(m+1)*theta*gamma*abs(q)^(m-1)*q/(4*epsilon*h^(m+1)*w^(m+2))*w_x^2 ...
            - m*theta*gamma*abs(q)^(m-1)*q/(4*epsilon*h^(m+1)*w^(m+1))*w_xx ...
            + theta*(fout(1,4,4)+b_xxx)/(4*epsilon) ...
            +(1-theta)*(1-r)*fout(1,4,4)/(4*epsilon) ...
            + h*w_xx*abs(sigma)^(n+1)/q - a_xx*w*sigma/q -  2*a_x*w_x*sigma/q - a*w_xx*sigma/q ...
            + sigma*w_xxx/w - sigma*w_xx*w_x/w^2 - 2*sigma*w_x*w_xx/w^2 + 2*sigma*w_x^3/w^3;
        
        fout(3,4,4) =  a_xx*w + 2*a_x*w_x +  a*w_xx;
       
    case 10
        
        type = parameters.type;
        fout = zeros(4,length(type));
        for ii = 1:length(type)
            switch type(ii)
                case 1
                    dada_0 = parameters.accfun(x_prime,0);
                    fout(1,ii) = h*w*dada_0/q;
                    fout(2,ii) = theta*fout(1,ii)/(4*epsilon) ...
                         + (1-theta)*(1-r)*fout(1,ii)/(4*epsilon) ...
                         -dada_0*w*sigma/q;
                    fout(3,ii) = dada_0*w;

                case 2
                    dgammadC_0 = epsilon^((n^2*(1+m)-n)/(n+1)^2);
                    fout(2,ii) = theta*dgammadC_0*abs(q)^(m-1)*q/(4*epsilon*h^(m+1)*w^m);
                    
                case 3
                    dmudM_0 = mu/M_0;
                    fout(1,ii) = -(1-theta)*h*w*dmudM_0/q;
                    fout(2,ii) = theta*fout(1,ii)/(4*epsilon) ...
                         + (1-theta)*(1-r)*fout(1,ii)/(4*epsilon) ...
                         + (1-theta)*dmudM_0*w*sigma/q;
                    fout(3,ii) = -(1-theta)*dmudM_0*w;

                case 5
                    dmudx_f = -epsilon^(-n/(n+1))*M_0*mu_x;
                    fout(1,ii) = (1-theta)*h*w*dmudx_f/q;
                    fout(2,ii) = (1-theta)*dmudx_f*w*sigma/q;
                    fout(3,ii) = -(1-theta)*dmudx_f*w;
                
            end
        end
        
    case 11
        
        type = parameters.type;        
        fout = zeros(4,4,length(type));
        for ii = 1:length(type)
            switch type(ii)
                case 1
                    dada_0 = parameters.accfun(x_prime,0);
                    d2ada_0dx = parameters.accfun(x_prime,1);
                    fout(1,1,ii) = w*dada_0/q;
                    fout(1,3,ii) = -h*w*dada_0/q^2;
                    fout(1,4,ii) = h*w_x*dada_0/q + h*w*d2ada_0dx/q;
                    fout(2,1,ii) = theta*fout(1,1,ii)/(4*epsilon) ...
                         + (1-theta)*(1-r)*fout(1,1,ii)/(4*epsilon);
                     fout(2,2,ii) = theta*fout(1,2,ii)/(4*epsilon) ...
                         + (1-theta)*(1-r)*fout(1,2,ii)/(4*epsilon)...
                         -dada_0*w/q;
                    fout(2,3,ii) = theta*fout(1,3,ii)/(4*epsilon) ...
                         + (1-theta)*(1-r)*fout(1,3,ii)/(4*epsilon)...
                         + dada_0*w*sigma/q^2;
                    fout(2,4,ii) = theta*fout(1,4,ii)/(4*epsilon) ...
                         + (1-theta)*(1-r)*fout(1,4,ii)/(4*epsilon)...
                         -dada_0*w_x*sigma/q - d2ada_0dx*w*sigma/q;
                    fout(3,4,ii) = dada_0*w_x + d2ada_0dx*w;
               
                case 2
                    dgammadC_0 = epsilon^((n^2*(1+m)-n)/(n+1)^2);
                    fout(2,1,ii) = -(m+1)*theta*dgammadC_0*abs(q)^(m-1)*q/(4*epsilon*h^(m+2)*w^m);
                    fout(2,3,ii) = m*theta*dgammadC_0*abs(q)^(m-1)/(4*epsilon*h^(m+1)*w^m);
                    fout(2,4,ii) = - m*theta*dgammadC_0*abs(q)^(m-1)*q/(4*epsilon*h^(m+1)*w^(m+1))*w_x;
                    
                case 3
                    dmudM_0 = mu/M_0;
                    d2mudM_0dx = mu_x/M_0;
                    fout(1,1,ii) = -(1-theta)*w*dmudM_0/q;
                    fout(1,3,ii) = (1-theta)*h*w*dmudM_0/q^2;
                    fout(1,4,ii) = -(1-theta)*h*w_x*dmudM_0/q - (1-theta)*h*w*d2mudM_0dx/q;
                    fout(2,1,ii) = theta*fout(1,1,ii)/(4*epsilon) ...
                         + (1-theta)*(1-r)*fout(1,1,ii)/(4*epsilon);
                     fout(2,2,ii) = theta*fout(1,2,ii)/(4*epsilon) ...
                         + (1-theta)*(1-r)*fout(1,2,ii)/(4*epsilon)...
                         + (1-theta)*dmudM_0*w/q;
                    fout(2,3,ii) = theta*fout(1,3,ii)/(4*epsilon) ...
                         + (1-theta)*(1-r)*fout(1,3,ii)/(4*epsilon)...
                         - (1-theta)*dmudM_0*w*sigma/q^2;
                    fout(2,4,ii) = theta*fout(1,4,ii)/(4*epsilon) ...
                         + (1-theta)*(1-r)*fout(1,4,ii)/(4*epsilon)...
                         + (1-theta)*dmudM_0*w_x*sigma/q + (1-theta)*d2mudM_0dx*w*sigma/q;
                    fout(3,4,ii) = -(1-theta)*dmudM_0*w_x - (1-theta)*d2mudM_0dx*w;
 
                    fout(3,4,ii) = -(1-theta)*dmudM_0*w_x -(1-theta)*d2mudM_0dx*w;

                case 5
                    dmudx_f = -epsilon^(-n/(n+1))*M_0*mu_x;
                    fout(1,1,ii) = (1-theta)*w*dmudx_f/q;
                    fout(1,3,ii) = -(1-theta)*h*w*dmudx_f/q^2;
                    fout(1,4,ii) = (1-theta)*h*w_x*dmudx_f/q;
                    fout(2,3,ii) = -(1-theta)*dmudx_f*w*sigma/q^2;
                    fout(2,4,ii) = (1-theta)*dmudx_f*w_x*sigma/q;
                    fout(3,4,ii) = -(1-theta)*dmudx_f*w_x;
                     
            end
        end        
end

if isfield(parameters,'reverse') && parameters.reverse
    fout = -fout;
end

end
