function fout = outlet_shoot(v_in,ds,scale,direction,npts,parameters)
%fout = outlet_shoot(v_in,ds,scale,direction,npts,parameters)
%   Solves the steady state problem for a marine-terminating outlet
%   glacier with melt and/or calving.
%
%   The steady state problem for a tidewater glacier with sidewall drag,
%   basal melting under its floating tongue and Nick et al (2010)-style
%   calving takes the form
%       4(h|u_x|^(1/n-1)u_x)_x - gamma|u|^(m-1)u - latdrag*w^(-1/n-1)h|u|^(1/n-1)u -
%       h*((1-(1-theta)*r)*h_x - theta*b_x) = 0
%       (uhw)_x = w*(a - (1-theta)*mu)
%   where b is bed elevation, a is surface accumulation, C_0 and mu are
%   dimensionless basal drag coefficient and basal
%   melt rate on the grounded and floating portions of the glacier tongue,
%   and theta is the indicator function for flotation
%       theta = 1 if h >= -b/r, theta = 0 otherwise
%   
%   Possible boundary conditions are either a calving boundary condition as
%   developed by Nick et al (2010), giving
%       4h|u_x|^(1/n-1)u_x = (1-(1-theta)*r)/2*h^2-theta*b^2/(2*r)
%       h = -b*phi(-r*lambda_0/b)
%   for a particular continuous function phi,
%        phi = 1 + (1-r)/r*zeta + sqrt((1+(1-r)/r*zeta)^2-1/r) if zeta < 1/2
%        phi = 2*zeta/r if zeta > 1/2,
%   and Lambda_0 is the dimensionless water filling level of ice surface
%   crevasses.
%   Alternatively, the terminus can be melt-controlled, with
%       4h|u_x|^(1/n-1)U_x -> 0
%       uh -> 0
%   as the terminus is approached, while at the ice divide x = 0
%       u = 0.
%
%   The problem can be re-written in the form of a first-order system by
%   putting
%       sigma = |u_x|^(1/n-1)*u_x
%       q = u*h*w
%   Then
%       h_x = h^2*w*|sigma|^(n-1)*sigma/q + h*w*a/q - h*w_x/w
%       sigma_x = 
%       sigma_x = latdrag*|q|^(1/n-1)*q/(4*epsilon*h^(1/n)*w^(1/n+1)) ...
%               + theta*ga,,a*|q|^(m-1)*q/(4*epsilon*h^(m+1)*w^m) ...
%               + ((1-(1-theta)*r)*h_x - theta*b_x)/(4*epsilon) ...
%               + h*w*|sigma|^(n+1)/q - w*a*sigma + sigma*w_x/w
%       q_x = w*(a - (1-theta)*mu)
%   with boundary conditions q = 0 at x = 0, and either
%       4*h*sigma = (1-(1-theta)*r)/2*h^2-theta*b^2/(2*r)
%       h = -b*phi(-r*lambda_0/b)
%   or
%       q -> 0, h*sigma -> 0
%   Note that the origin x = 0 is a singular point, near which the system
%   behaves as q -> 0, sigma -> sigma_ 0 := (a_0/h_0)^(1/n), h -> h_0, with
%   h_0 to be determined, and a_0 = lim_{x -> 0^+) a(x); the first order
%   correction term takes the form
%       h_1 ~ -b_1 + 4*epsilon*sigma_0/a_0*a_1 + 4*epsilon*sigma_0/w_0*w_1,
%       sigma_1 ~ - sigma_0/(n*h_0)*h_1 - sigma_0/(n*w_0)*w_1
%       q_1 ~ a_0*w_0*x
%   where b_1, a_1 and w_1 are linear corrections to bed elevation,
%   accumulation rate and channel width at the origin.
%
%   By a coordinate transform dx/dzeta = q, this can be turned into a
%   fixed point problem in which the fixed point has a stable and unstable
%   manifold as well as a centre manifold corresponding to other fixed
%   points (and therefore with no dynamics on the centre manifold); the
%   orbit into the fixed point then corresponds to the unstable manifold
%   and is unique. The key is to determine *which* fixed point permits the
%   far field (terminus) conditions to be satisfied.
%
%   The fixed point has a single degree of freedom (h_0) to be solved for,
%   and this is done here by means of a single shooting problem, tracing
%   the solution away from the origin for a given h_0 and attempting to
%   satisfy the glacier terminus conditions; one condition is applied as an
%   event function determining terminus location, the other as a constraint
%   on h_0.
%
%   In the case of a melt-limited terminus, the terminus itself is also
%   singular. Here, h -> 0 and therefore q ~ (mu-a)*(x_c-x)-> while u
%   remains bounded away from zero, where x_c is the terminus location. In
%   terms of the phase variables we have,
%       sigma ~ 1/(8*epsilon)*((1-r) - (u_c^(1/n+1)/(w^(1/n+1)*(mu-a)))*(mu-a)*(x_c-x)/u_c
%       q ~ (mu-a)*(x_c-x)
%       h ~ q/u_c
%   where u_c is the limiting value of the velocity at the terminus. Note
%   that the behaviour near the singular point at the terminus is then
%   uniquely determined by u_c, just as behaviour near the ice divide is
%   uniquley determined by h_0.
%
%   Now, it becomes necessary to shoot from both, the ice divide and
%   the terminus to the grounding line. There both solutions must match,
%   and thickness must equal flotation; one of these four conditions serves
%   to determine grounding line location / terminus while the other two
%   determine u_c and h_0.
%
%   Computationally, we permit mu, a, b and w to be functions. In order to
%   make comparison with the boundary layer theory simpler, mu and b are
%   specified in scaled form, as functions meltfun(x-x_f) and Bfun(x), related to mu and
%   b through
%       mu(x) = epsilon^(-n/(n+1))*M_0*meltfun(x-x_f)
%       b(x) =  epsilon^(n^2/(n+1)^2)*Bfun(x)
%   where x_f is the grounding line position, melt being assumed to depend
%   on distance from the grounding line (if at all), and M_0 is an O(1)
%   scaling factor. a is prescribed through a function accfun(x) and a scale
%   factor a_0 through
%       a(x) = a_0*accfun(x),
%   while w(x) = wfun(x) without any scaling parameters.
%
%   Similarly, the parameters gamma and lambda are specified in scaled
%   form as parameters C_0 and Lambda_0, related through
%       gamma = epsilon^((n^2*(1+m)-n)/(n+1)^2)*C_0
%       lambda = epsilon^(-n^2/(n+1)^2)*r*Lambda_0
%   B = Bfun(x_f), C_0, M_0 and Lambda_0 appear directly in the boundary
%   layer theory; rhe boundary layer theory assumes m_0 = 1, but we permit
%   m_0 to vary here for additional flexibility in modelling.
%
%   Input arguments:
%       v_in:   starting guess for a pair [h_0 indvar], where indvar is the
%               forcing parameter is being varied, or
%               'calving_type' is set to 'grounded calving' or 'floating
%               calving' (see under 'parameters' below), or a quadruple
%               [h_0;u_c;L_c;indvar] if 'calving_type' is set to
%               'melt_limit'. Here L_c = x_c-x_f; it is easier to work with
%               L_c than x_c because initialising L_c requires only a good
%               guess of shelf length, not of grounding line position.
%       ds:     step size for arc length continuation method (see
%               documentation for Arclength_single_symmetric_v1), default
%               1e-2
%       scale:  defines a linear ('linear') or logarithmic ('log')
%               definition of arc length size (see
%               documentation for Arclength_single_symmetric_v1)
%       direction: defines direction in which arc length algorithm
%               progresses from initial point (default +1, meaning the
%               variable specified by 'indvar' is to hincrease)
%       npts:   number of points that the arc length continuation code
%               tries to compute (default 2)
%       parameters: parameter structure, with optional fields
%           indvar: string argument, defines whether the code solves for
%               h_0 (and u_c, x_c if calving_type is set to 'melt limit',
%               see below) in terms of a_0 ('a'), C_0 ('C'), M_0 ('M'),
%               Lambda_0 ('Lambda'). Defaults to 'a'
%           model:  model parameter substructure containing subfields
%               m,n,r, epsilon C_0, M_0, Lambda_0, a_0, latdrag. All
%               default to except n = 1/m = 3, r = 0.9, epsilon = 1e-2
%               In addition, parameters.model contains function handles Bfun,
%               Wfun, accfun, meltfun as defined above. The functions take
%               the form accfun(x,ndiff,parameters.model),
%               meltfun(x-x_f,ndiff,paramteers.model),
%               Bfun(x,ndiff,parameters.model),
%               Wfun(x,ndiff,parameters.model) where x_f is grounding line
%               position and ndiff is differentiation level (ndiff = 0 is
%               the required function, ndiff = 1 gives its first derivative
%               etc.) accfun and meltfun need to be defined up to ndiff =
%               2, Bfun and Wfun up to ndiff = 3.                   
%           calving_type: Defines the style of calving to be solved for.
%               Options are the following strings
%                   'melt limit': no calving, physically - but not
%                   mathematically - equivalent to Lambda = 0 in Nick et al
%                   law
%                   'grounded calving': calving according to Nick et al
%                   law, terminus is assumed to be grounded
%                   'floating calving': calving according to Nick et al
%                   law, terminus is assumed to be afloat
%               Default is 'melt limit'
%               Note that the code as it stands does not deterine
%               automatically whether the calving front should be afloat or
%               grounded, this is a desirable improvement for future
%               versions.
%           perturb_fixedpoint: technical parameter defining the size of
%               perturbation away from each fixed point used to compute the
%               orbit 'out of the fixed point', default 1e-3
%           t_span: technical parameter defining the maximum integration
%               interval from the starting point of each orbit, default
%               1000.  Note that it is possible to provide initial guesses
%               for which the code will exit with an error, namely where a
%               require event function never goes to zero; t_span ensures
%               that the code will not run indefinitely in that case.
%           solver: defines matlab ode solver and tolerances to use, with
%               subfields (see documentation for shoot_parameters)
%                   method: ode solver, choices are '15s', '23t', '45',
%                   default is '15s' (as the problem is stiff for small
%                   epsilon)
%                   AbsTol: absolute tolerance, default is 1e-12
%                   RelTol: relative tolerance, default is 1e-12
%           functions: substructure defining routines @evolve,
%               @event and @parset used by shoot_parameter (not
%               user-definable) as well as the calving function through
%               subfield 'calve'. The calving function can be defined by
%               the user (default is @Nick_calve, other option is
%               @float_calve, which can only be used in conjunction with
%               calving_type 'grounded calving'. The calving function must
%               take the form
%                   calve_phi(zeta,ndiff,floatspec,parameters)
%               where
%                   zeta = Lambda/H_g
%                   n_diff = order of differentiation of phi (0 or 1)
%                   floatspec = 'grounded' for grounded calving, 'floating'
%                   for floating calving
%                   parameters = the parameters.model substructure defined
%                   above
%               and calve_phi is the fraction of the depth to bedrock r*H_g
%               attained by ice thickness at the calving front
%           srchparams: search parameters that are passed on onto arc
%               length continuation code ArcLength_symmetric_v1; default is
%               to use default setting for that code (see documentation for
%               ArcLength_symmetric_v1)
%
%   Output argument:
%       fout:   solution structure with fields
%           Q_g: npts-by-1 vector of values of Q_g
%           a_0, C_0, M_0 or Lambda_0: vector of corresponding
%               values of the 'independent variable' defined by indvar
%           L_c, u_c: the values of L_c at the glacier terminus, only output if
%               calving_type is set to 'melt limit'
%           error_flag = npts-by-1 vector of booleans indicating whether
%                   the Newton solver has converged
%           parameters: the full parameter structure used in the solution
%                   algorithm (including all default and internal settings)
%       
%   Required subroutines: 
%       Arclength_single_symmetric_v1: basic solver
%       shoot_parameters:    basic shooting method solver, computes Jacobian
%                           with respect to model parameters as well as
%                           initial conditions
%       outlet_constraint:  wrapper for shooting method solver,
%                           applies boundary conditions.
%       outlet_evolve:      The right-hand side of the dynamical system
%                           above, plus its derivative arrays
%       outlet_ event:      The event function that defines, variously,
%                           thickness at flotation or stress at a grounded
%                           or floating glacier terminus, plus the
%                           derivatives of that event function
%       outlet_parset:      Rerouting routine that replaces appropriate
%                           entries in the parameters structure, as
%                           required by shoot_parameters
%       Nick_calve or float_calve:  defines calving function
%       accfun, meltfun, Bfun, Wfun: subroutines defining surface
%                           accumulation, shelf basal melting and channel
%                           geometry
%
%
%Christian Schoof, July 2016


if nargin < 2 || isempty(ds)
    ds =1e-2;
end
if nargin < 3 || isempty(scale)
    scale ='linear';
end
if nargin < 4 || isempty(direction)
    direction = +1;
end
if nargin < 5 || isempty(npts)
    npts = 2;
end

%Define the independent variable
if ~isfield(parameters,'indvar')
    parameters.indvar = 'a';
end

%Define the type of problem to be solved
if ~isfield(parameters,'calving_type')
    parameters.calving_type = 'melt limit';
    %parameters.calving_type = 'grounded calving';
    %parameters.calving_type = 'floating calving';
end

%calving law
if ~isfield(parameters,'functions') || ~isfield(parameters.functions,'calve')
    parameters.functions.calve = @Nick_calve;   
    %parameters.functions.calve =  @float_calve;
end

%fixed parameters
if nargin < 6 || ~isfield(parameters,'model') || ~isfield(parameters.model,'n')
    parameters.model.n = 3;
end
if ~isfield(parameters.model,'m')
    parameters.model.m = 1/3;
end
if ~isfield(parameters.model,'r')
    parameters.model.r = 0.9;
end
if ~isfield(parameters.model,'epsilon')
    parameters.model.epsilon = 1e-2;
end
if ~isfield(parameters.model,'latdrag')
    parameters.model.latdrag = 1;
end
%parameters that can be varied as forcing parameters
if ~isfield(parameters.model,'C_0')
    parameters.model.C_0 = 1;
end
if ~isfield(parameters.model,'M_0')
    parameters.model.M_0 = 1;
end
if ~isfield(parameters.model,'Lambda_0')
    parameters.model.Lambda_0 = 1;
end
if ~isfield(parameters.model,'a_0')
    parameters.model.a_0 = 1;
end
%default model functions
if ~isfield(parameters.model,'accfun')
    parameters.model.accfun = @accumulation;
end
if ~isfield(parameters.model,'meltfun')
    parameters.model.meltfun = @melting;
end
if ~isfield(parameters.model,'Bfun')
    parameters.model.Bfun = @bedelevation;
end
if ~isfield(parameters.model,'Wfun')
    parameters.model.Wfun = @channelwidth;
end
%Set flotation indicator function theta for test purposes only
if ~isfield(parameters.model,'theta')
    parameters.model.theta = 1;
end

%computational parameter defining initial perturbation from fixed point
if ~isfield(parameters,'perturb_fixedpoint')
    parameters.perturb_fixedpoint = 1e-3;
end

if nargin == 0 || ~isfield(parameters,'t_span')
    parameters.t_span = [0 1000];
end

if ~isfield(parameters,'solver') || ~isfield(parameters.solver,'method')
    parameters.solver.method = '15s';
end

%fixed functions and other non-user definable parameters
parameters.functions.evolve = @outlet_evolve;
%DEBUG: uncomment next line for correct version
parameters.functions.events = @outlet_event;
parameters.functions.parset = @outlet_parset;

switch parameters.indvar
    case 'a'
        parameters.model.type = 1;
    case 'C'
        parameters.model.type = 2;
    case 'M'
        parameters.model.type = 3;
    case 'Lambda'
        parameters.model.type = 4;
    case 'x_f'
        if ~isfield(parameters,'flags') || ~isfield(parameters.flags,'test') || ~parameters.flags.test
            error('x_f is an invalid independent variable option except in test mode')
        else
            parameters.model.type = 5;
        end
    otherwise
        error('Invalid independent variable option')
end

parameters.ndegf = 4;



%set initial conditions for solver here (v_0); v_in is used for testing
%routine
if nargin == 0 || isempty(v_in)
    v_in = [];
    switch parameters.calving_type
        case 'melt limit'
            v_0 = [.1; .6; 1; 1];
        case 'floating calving'
             v_0 = [1; 6*(1-parameters.model.r)^(-parameters.model.n^2/(parameters.model.n+1)^2)];
        case 'grounded calving'
             v_0 = [.025;1];
    end
else
    v_0 = v_in;
end

%The field bc_type set here is only used for debugging,
%boundary_layer_constraint will override this
if ~isfield(parameters.model,'bc_type')
    %parameters.model.bc_type = 'grounded calving';
    %parameters.model.bc_type = 'flotation';
    parameters.model.bc_type = 'grounding';
end

%Debug tests, triggered by parameters.flags.test. See Jacobian_test_single and
%shoot_parameters documentation for details
if isfield(parameters,'flags') && isfield(parameters.flags,'test') && parameters.flags.test
    if ~isfield(parameters.flags,'constraint_only') || ~ parameters.flags.constraint_only
        if isempty(v_in) || length(v_in) ~= 5
            v_rand = rand(5,1);
            switch parameters.model.bc_type
                case 'flotation'
                    v_rand(1:4) = [.01 1 1 2].';
                case 'grounded calving'
                    v_rand(1:4) = [.01 1 1 2].';
                otherwise
                    parameters.model.reverse = true;
                    parameters.model.theta = 1/2;
                    v_rand(1:4) = [1 1 1 2].';
            end
        else
            v_rand = v_in;
        end
        if ~isfield(parameters.flags,'derivative_test'), parameters.flags.derivative_test = true; end
        fout.derivative_test = shoot_parameters(v_rand,parameters);
        parameters.flags.test = false;
        parameters.flags.derivative_test = false;
        fout.shoot_test = Jacobian_test_single(@shoot_parameters,v_rand,parameters,1e-6);
        if isfield(parameters.flags,'evolve_only') && parameters.flags.evolve_only
            fout.parameters = parameters;
            return
        end
        parameters.model.reverse = false;
    end
    if isempty(v_in) || length(v_in) == 5, 
        switch parameters.calving_type
            case 'melt limit'
                parameters.indvar = 'M';
                v_in = [10;20;0.1;1];
            case 'floating calving'
                v_in = [10; 1];
            case 'grounded calving'
                v_in = [5; .5];
        end
    end
    fout.constraint_test = Jacobian_test_single(@outlet_constraint,v_in,parameters,1e-6);
    fout.parameters = parameters;
    return
end

%output: solve boundary layer problem
if isfield(parameters,'srchparams')
    [ftemp, errors] = ArcLength_single_symmetric_v1(@outlet_constraint,v_0,ds,[],scale,direction,npts,parameters,parameters.srchparams);
else
    [ftemp, errors] = ArcLength_single_symmetric_v1(@outlet_constraint,v_0,ds,[],scale,direction,npts,parameters);
end
fout.h_0 = ftemp(1,:).';
switch parameters.indvar
    case 'a'
        switch parameters.calving_type
            case 'melt limit'
                fout.a_0 = ftemp(4,:).';
            otherwise
                fout.a_0 = ftemp(2,:).';
        end
    case 'C'
        switch parameters.calving_type
            case 'melt limit'
                fout.C_0 = ftemp(4,:).';
            otherwise
                fout.C_0 = ftemp(2,:).';
        end
    case 'M'
        switch parameters.calving_type
            case 'melt limit'
                fout.M_0 = ftemp(4,:).';
            otherwise
                fout.M_0 = ftemp(2,:).';
        end
    case 'Lambda'
        switch parameters.calving_type
            case 'melt limit'
                fout.Lambda_0 = ftemp(4,:).';
            otherwise
                fout.Lambda_0 = ftemp(2,:).';
        end        
end
if strcmp(parameters.calving_type,'melt limit')
    fout.u_c = ftemp(2,:).';
    fout.L_c = ftemp(3,:).';
end
fout.error_flag = errors;
fout.parameters = parameters;

end
